/*
 * Routines to check for the occurrence of a class.
 *
 * Author: Bert Bos <bert@w3.org>
 * Created: 20 Aug 2000
 * Version: $Id: class.c,v 1.2 2005/04/29 22:51:11 bbos Exp $
 *
 **/

#include "config.h"
#include <assert.h>
#if STDC_HEADERS
# include <string.h>
#else
# ifndef HAVE_STRCHR
#  define strchr index
#  define strrchr rindex
# endif
# ifndef HAVE_STRSTR
#  include "strstr.e"
# endif
#endif
#include <ctype.h>
#include "export.h"
#include "types.e"


/* contains -- check if string contains a certain word, return pointer */
EXPORT const string contains(const string s, const string word)
{
  string t = s;
  unsigned char c;

  while ((t = strstr(t, word))) {
    if ((c = *(t + strlen(word))) && !isspace(c)) t++; /* Not end of word */
    else if (t != s && !isspace(*(t - 1))) t++;	/* Not beginning of word */
    else return t;				/* Found it */
  }
  return NULL;					/* Not found */
}

/* has_class -- check for class=word in list of attributes */
EXPORT Boolean has_class(pairlist attribs, const string word)
{
  pairlist p;

  for (p = attribs; p; p = p->next) {
    if (strcasecmp(p->name, "class") == 0 && contains(p->value, word))
      return True;
  }
  return False;
}

/* in_list -- check if word occurs in array list */
static Boolean in_list(const string word, const string *list)
{
  int i;

  for (i = 0; list[i]; i++) if (contains(word, list[i])) return True;
  return False;
}

/* has_class_in_list -- check for class=word for all words in array c */
EXPORT Boolean has_class_in_list(const pairlist attribs, const string *c)
{
  pairlist p;

  assert(c);
  for (p = attribs; p; p = p->next) {
    if (strcasecmp(p->name, "class") == 0 && in_list(p->value, c))
      return True;
  }
  return False;
}


